/*
www.UltimateGameProgramming.com
PerPixel Lighting Shader in GLSlang.
Created by the Programming Ace.


Varying...
texCoord0 = output vertex texture coordinates.
normal = vertex normal.
pos = vertex pos.


Uniform...
decal = binded texture.
lightPos = light position.
lightColor = light color.
ambientColor = ambient color.
matAmbient = ambient material.
matDiffuse = diffuse material.
matSpecular = specular material.
emissive = emissive contribution.
shininess = how shinny object is.
cameraPos = view position.


Output...
gl_FragColor = output vertex color.
*/

varying vec2 texCoord;
//varying vec4 shadowTexCoord;
varying vec3 normal;
varying vec4 pos;

uniform sampler2D decal;
//uniform sampler2DShadow shadowMap;

void main()
{
	// Re-normalize.
	vec3 n = normalize(normal);

	vec4 emissive = vec4(0,0,0,0); //gl_FrontMaterial.emissive;

	// Get ambient.
	vec4 ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;

	// Get diffuse.
	vec4 lightVec = gl_LightSource[0].position;//normalize(gl_LightSource[0].position - pos);
	float diffuseLight = max(dot(n, lightVec.xyz), 0.0) ;
	vec4 diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse * diffuseLight;

	// Get specular.
	vec4 eyeVec = normalize(vec4(0,0,0,0) - pos);
	//vec4 halfVec = normalize(lightVec + eyeVec);
	vec4 halfVec = gl_LightSource[0].halfVector;
	float shininess = max(gl_FrontMaterial.shininess, 0.0);
	//shininess = 50;
	float specularLight = pow(max(dot(gl_NormalMatrix * n, halfVec.xyz), 0.0), shininess);

	// If there's no diffuse light, turn off the specular
	if(diffuseLight <= 0.0) specularLight = 0.0;
	vec4 specular = gl_FrontMaterial.specular * specularLight;

	// Shadow
	//float shadow;
	//shadow = shadow2DProj(shadowMap, shadowTexCoord).r;
	/*
	float pseudorandom=1.0;
	float shadow = 0.0;
	
	vec4 newShadowTexCoord;
	
	for(int i=0; i<4; i++){
		newShadowTexCoord = shadowTexCoord;
		pseudorandom = mod((pos.x*591.0+pos.y*113.0+pos.z*627.0)*475561.0*(pseudorandom+1.0),10000.0)/5000.0-1.0;
		newShadowTexCoord.x+=pseudorandom/64.0;
		
		pseudorandom = mod((pos.x*561.0+pos.y*123.0+pos.z*657.0)*474561.0*(pseudorandom+1.0),10000.0)/5000.0-1.0;
		newShadowTexCoord.y+=pseudorandom/64.0;
		
		shadow += shadow2DProj(shadowMap, newShadowTexCoord).r*0.25;
	}
	*/

	// Texture
	vec4 texel = texture2D(decal, texCoord);

	// Final lighting color.
	//gl_FragColor = (emissive + ambient + diffuse * (shadow * 0.9 + 0.1)) * texel + specular * shadow;
	gl_FragColor = (emissive + ambient + diffuse) * texel;// + specular; //(emissive + ambient + diffuse) * texel + specular;
	
	gl_FragColor.w = 1.0;

	//gl_FragColor[0] = (1-texture2DProj(shadowMap, shadowTexCoord).r) * 30;
	//gl_FragColor[1] = gl_FragColor[0];
	//gl_FragColor[2] = gl_FragColor[0];
}
