attribute vec2 I0;
attribute vec4 WP0;
attribute vec4 WP1;
attribute vec4 WNU0;
attribute vec4 WNU1;
attribute vec4 WNV0;
attribute vec4 WNV1;
attribute vec2 TEX;

varying vec3 normal;
varying vec2 texCoord;
//varying vec4 shadowTexCoord;
varying vec4 pos;

uniform sampler2D particleTexture;

#define textureSize 128.0
#define texelSize 1.0 / 128.0

/// Trilinear lattice deformer using position and deformation gradient-based tangent space weights. 
/// Doug L. James, 1/12/07
void main() 
{
	/// READ IN OWNER CELL DATA FROM A TEXTURE
	vec4 d0 = texture2D(particleTexture, I0 );
	vec4 d1 = texture2D(particleTexture, I0 + vec2(texelSize, 0));
	vec4 d2 = texture2D(particleTexture, I0 + vec2(texelSize * 2, 0));
	vec4 d3 = texture2D(particleTexture, I0 + vec2(texelSize * 3, 0));
	vec4 d4 = texture2D(particleTexture, I0 + vec2(texelSize * 4, 0));
	vec4 d5 = texture2D(particleTexture, I0 + vec2(texelSize * 5, 0));

	// Move packed particle data into unpacked vectors for each particle
	vec4 p0 = vec4(d0.r, d0.g, d0.b, 0);
	vec4 p1 = vec4(d0.a, d1.r, d1.g, 0);
	vec4 p2 = vec4(d1.b, d1.a, d2.r, 0);
	vec4 p3 = vec4(d2.g, d2.b, d2.a, 0);
	vec4 p4 = vec4(d3.r, d3.g, d3.b, 0);
	vec4 p5 = vec4(d3.a, d4.r, d4.g, 0);
	vec4 p6 = vec4(d4.b, d4.a, d5.r, 0);
	vec4 p7 = vec4(d5.g, d5.b, d5.a, 0);
	
	//p0 = vec4(-0.5, 0, -10.5,0);
	//p1 = vec4(-0.5, 0, -9.5,0);
	//p2 = vec4(-0.5, 1, -10.5,0);
	//p3 = vec4(-0.5, 1, -9.5,0);
	//p4 = vec4(0.5, 0, -10.5,0);
	//p5 = vec4(0.5, 0, -9.5,0);
	//p6 = vec4(0.5, 1, -10.5,0);
	//p7 = vec4(0.5, 1, -9.5,0);
	
	/// WEIGHTED POSITION:
	vec4 p      = WP0[0]*p0 + WP0[1]*p1 + WP0[2]*p2 + WP0[3]*p3 + WP1[0]*p4 + WP1[1]*p5 + WP1[2]*p6 + WP1[3]*p7;
	pos			= vec4(p[0], p[1], p[2], 1);
	vec4 pos2   = gl_ModelViewMatrix * pos;
	gl_Position = gl_ProjectionMatrix * pos2;

	/// WEIGHTED TANGENTS:
	vec4   u  = WNU0[0]*p0 + WNU0[1]*p1 + WNU0[2]*p2 + WNU0[3]*p3 + WNU1[0]*p4 + WNU1[1]*p5 + WNU1[2]*p6 + WNU1[3]*p7;
	vec4   v  = WNV0[0]*p0 + WNV0[1]*p1 + WNV0[2]*p2 + WNV0[3]*p3 + WNV1[0]*p4 + WNV1[1]*p5 + WNV1[2]*p6 + WNV1[3]*p7;
	vec3   n  = normalize(cross(vec3(u[0], u[1], u[2]), vec3(v[0], v[1], v[2])));

	normal    = n;

	// TEXTURE STUFF:
	texCoord = TEX;
	//shadowTexCoord = gl_TextureMatrix[1] * pos;
}
